@@ -1,10 +1,11 @@
+import datetime
 from fabric.api import *
 from fabric.contrib.files import exists
 import os
 from .tasks import restart
 from .utils import upload_dir
 
-__all__ = ['deploy', 'build', 'upload']
+__all__ = ['deploy', 'build', 'upload', 'versions', 'update_version']
 
 # The definition of your apps as a dictionary. This can have four keys:
 #
@@ -32,8 +33,15 @@ def deploy(app, commit='origin/master'):
     """
     Build and upload an app.
     """
+    kwargs = {}
+    # If no hosts have been set by the user, default to this app's role
+    if not env.hosts:
+        kwargs['role'] = env.apps[app]['role']
+
     execute(build, app, commit)
-    execute(upload, app, role=env.apps[app]['role'])
+    version = datetime.datetime.now().replace(microsecond=0).isoformat().replace(':', '-')
+    execute(upload, app, version, **kwargs) 
+    execute(update_version, app, version, **kwargs)
 
 @task
 @runs_once
@@ -60,38 +68,56 @@ def build(app, commit='origin/master'):
         with lcd(path):
             local(env.apps[app]['build'])
 
+def _versions(app):
+    return sudo('ls "%s"' % os.path.join(env.app_root, app+'-versions')).split()
+
+def _current_version_path(app):
+    return sudo('readlink "%s"' % os.path.join(env.app_root, app))
+
 @task
-@parallel
-def upload(app):
+def versions(app):
     """
-    Upload the code for an app and restart its init script
+    Print the versions of an app that are available
+    """
+    print '\n'.join(_versions(app))
+
+@task
+def update_version(app, version):
+    """
+    Switch the symlink for an app to point at a new version and restart its init script
     """
     symlink = os.path.join(env.app_root, app)
-    previous = os.path.join(env.app_root, app+'-previous')
-    current = os.path.join(env.app_root, app+'-current')
-
-    # Remove previous code
-    sudo('rm -rf "%s"' % previous)
-
-    # Move current code to previous
-    if exists(current):
-        sudo('cp -al "%s" "%s"' % (current, previous))
-        sudo('ln -sfn "%s" "%s"' % (previous, symlink))
-
-    # Upload new code
-    upload_dir('build/%s/*' % app, current, use_sudo=True)
-
-    # Run post-upload
-    if env.apps[app].get('post-upload'):
-        with cd(current):
-            sudo(env.apps[app].get('post-upload'))
-
-    # Swap!
-    sudo('ln -sfn "%s" "%s"' % (current, symlink))
+    version_path = os.path.join(env.app_root, app+'-versions', version)
+    sudo('ln -sfn "%s" "%s"' % (version_path, symlink))
 
     # Restart with upstart
     if env.apps[app].get('init'):
         restart(env.apps[app]['init'])
 
+@task
+@parallel
+def upload(app, version):
+    """
+    Upload the code for a version
+    """
+    all_versions_path = os.path.join(env.app_root, app+'-versions')
+    current_version_path = _current_version_path(app)
+    version_path = os.path.join(all_versions_path, version)
+
+    if not exists(all_versions_path):
+        sudo('mkdir "%s"' % all_versions_path)
+
+    # Copy existing code if it exists
+    if exists(os.path.join(env.app_root, app)):
+        sudo('cp -a "%s" "%s"' % (current_version_path, version_path))
+
+    # Upload new code
+    upload_dir('build/%s/*' % app, version_path, use_sudo=True)
+
+    # Run post-upload
+    if env.apps[app].get('post-upload'):
+        with cd(version_path):
+            sudo(env.apps[app].get('post-upload'))
+
 
 
