@@ -1,4 +1,5 @@
 from fabric.api import *
+from fabric.network import parse_host_string
 import subprocess
 
 __all__ = ['ssh', 'all', 'uptime', 'upgrade', 'restart', 'reboot']
@@ -8,24 +9,27 @@ def all():
     """
     Select all hosts
     """
-    env.hosts = []
+    host_set = set()
     for hosts in env.roledefs.values():
-        env.hosts.extend(hosts)
+        host_set.update(hosts)
     # remove dupes
-    env.hosts = list(set(env.hosts))
+    env.hosts = list(host_set)
 
 @task
 def uptime():
     run('uptime')
 
 @task
-def upgrade():
+def upgrade(non_interactive=False):
     """
     Upgrade apt packages
     """
     with settings(hide('stdout'), show('running')):
         sudo('apt-get update')
-    sudo("apt-get upgrade")
+    upgrade_command = ['apt-get', 'upgrade']
+    if non_interactive:
+        upgrade_command.append('-y')
+    sudo(' '.join(upgrade_command))
 
 @task
 def ssh(*cmd):
@@ -35,7 +39,11 @@ def ssh(*cmd):
     run = ['ssh', '-A', '-t']
     if env.key_filename:
         run.extend(["-i", env.key_filename])
-    run.append('%s@%s' % (env.user, env.host_string))
+    parsed = parse_host_string(env.host_string)
+    if parsed['port']:
+        run.extend(['-p', parsed['port']])
+    user = parsed['user'] if parsed['user'] else env.user
+    run.append('%s@%s' % (parsed['user'] if parsed['user'] else env.user, parsed['host']))
     run += cmd
     subprocess.call(run)
 
